@@ -510,7 +510,21 @@ class SplitTagger(ModelObject):
     @reconciled.on_first_get
     def reconciled_get(self):
         return self.tagRec in self.split.tags
-    
+
+    debit = UserAttr(AttrNumeric(2), "Debit", readonly=True)
+    @debit.on_first_get
+    def debit_get(self):
+        if self.split.sum > 0:
+            return AttrNumeric(2)(self.split.sum)
+        return AttrNumeric(2)(0)
+
+    credit = UserAttr(AttrNumeric(2), "Credit", readonly=True)
+    @credit.on_first_get
+    def credit_get(self):
+        if self.split.sum < 0:
+            return AttrNumeric(2)(-self.split.sum)
+        return AttrNumeric(2)(0)
+
     @classEvents.add("set", "reconciled")
     def reconciled_set(self, attr, oldvalue):
         linked_now = self.tagRec in self.split.tags
