@@ -10,6 +10,7 @@ import sys
 from qtalchemy import *
 from qtalchemy.xplatform import *
 from qtalchemy.widgets import *
+from qtalchemy.ext.reporttools import *
 from PyQt4 import QtCore, QtGui
 import datetime
 import csv
@@ -42,53 +43,21 @@ class Attr:
         for k,v in kwargs.iteritems():
             setattr(self,k,v)
 
-def ReportLabSize(str_pagesize):
-    """
-    Accepts input of a string "encoded" pagesize and translates it to a reportlab pagesize value.
-    
-    Example:
-    >>> ReportLabSize("letter")
-    (612.0, 792.0)
-    >>> ReportLabSize("legal")
-    (612.0, 1008.0)
-    >>> # a4 page dimensions are not rounded the same on diverse platforms
-    >>> tuple(AttrNumeric(2)(x) for x in ReportLabSize("landscape_a4"))
-    (Decimal('841.89'), Decimal('595.28'))
-    >>> ReportLabSize("landscape_letter")
-    (792.0, 612.0)
-    >>> ReportLabSize("portrait_letter")==ReportLabSize("letter")
-    True
-    """
-    from reportlab.lib import pagesizes
-
-    if str_pagesize is None:
-        mod, size = "", "letter" # sorry, I'm very North American
-    elif '_' in str_pagesize:
-        mod, size = str_pagesize.split('_',2)
-    else:
-        mod, size = "", str_pagesize
-    if not size.upper() in ("LETTER","LEGAL","ELEVENSEVENTEEN") + tuple("A{0}".format(i) for i in range(7)) + tuple("B{0}".format(i) for i in range(7)):
-        raise ValueError( "the pagesize {0} is not valid".format(size) )
-    if not mod.lower() in ("landscape","portrait",""):
-        raise ValueError( "the pagesize modifier {0} is not valid".format(mod) )
-
-    mySize = getattr(pagesizes,size.upper())
-    if mod != "":
-        mySize = getattr(pagesizes,mod.lower())(mySize)
-    return mySize
-
-def SuppressedZero(attr):
-    return lambda instance: getattr(instance, attr) if getattr(instance, attr) != 0 else ""
-
 def GeraldoTemplate(rpt, pagesize):
     from geraldo import Report, DetailBand, ObjectValue,FIELD_ACTION_COUNT, FIELD_ACTION_SUM,  ReportGroup,  ReportBand, SystemField, BAND_WIDTH, Label
     from geraldo.utils import cm, inch
     from geraldo.generators import PDFGenerator
     from reportlab.lib.enums import TA_CENTER, TA_RIGHT, TA_LEFT
 
+    if pagesize is None:
+        pagesize = rpt.page_size
+
+    if not isinstance(pagesize, tuple):
+        pagesize = ReportLabSize(pagesize)
+
     class Template(Report):
-        title = rpt.name
-        page_size = ReportLabSize(pagesize)
+        title = cgi.escape(rpt.report_title)
+        page_size = pagesize
         margin_top = .75*inch
         margin_bottom = .75*inch
         margin_left = .75*inch
@@ -135,6 +104,14 @@ class PyHaccReport(ModelObject):
         else:
             return re.sub("[^A-Za-z0-9]", "_", self.__class__.__name__)
 
+    @property
+    def report_title(self):
+        return self.name
+
+    @property
+    def page_size(self):
+        return "letter"
+
     def objectConverter(self):
         return None
 
@@ -207,8 +184,9 @@ class PyHaccReport(ModelObject):
                 import tempfile
                 import os
                 tempdir = tempfile.mkdtemp()
-                self.report.geraldo(os.path.join(tempdir,self.report.filename), None)
-                xdg_open(os.path.join(tempdir,self.report.filename))
+                fullpath = os.path.join(tempdir, self.report.filename + ".pdf")
+                self.report.geraldo(fullpath, None, detailRatios = [self.table.columnWidth(i) for i in range(self.table.model().columnCount(None))])
+                xdg_open(fullpath)
 
         outer = PromptWidget(parent, self)
         return outer
@@ -254,22 +232,53 @@ class PyHaccReport(ModelObject):
         doc = SimpleDocTemplate(fName,pagesize = ReportLabSize(pagesize))
         doc.build(story)
 
-    def geraldo(self, fName, pagesize):
-        from geraldo import Report, DetailBand, ObjectValue,FIELD_ACTION_COUNT, FIELD_ACTION_SUM,  ReportGroup,  ReportBand, SystemField, BAND_WIDTH
-        from geraldo.utils import cm
+    def geraldo(self, fName, pagesize, **kwargs):
+        from geraldo import Report, DetailBand, ObjectValue, FIELD_ACTION_COUNT, FIELD_ACTION_SUM, ReportGroup, ReportBand, SystemField, BAND_WIDTH
+        from geraldo.utils import inch
         from geraldo.generators import PDFGenerator
         from reportlab.lib.enums import TA_CENTER, TA_RIGHT
 
+        if pagesize is None:
+            pagesize = self.page_size
+        pagesize = ReportLabSize(pagesize)
+
         cols = self.columns()
+
+        if not kwargs.has_key("detailRatios"):
+            dr = [m.width for m in cols]
+            if set(dr) != set([None]):
+                kwargs['detailRatios'] = dr
+
+        if kwargs.has_key("detailRatios"):
+            detailRatios = kwargs["detailRatios"]
+            widths = sum(detailRatios)
+            for i in range(len(cols)):
+                detailRatios[i] = detailRatios[i] * (pagesize[0] - 1.5*inch) / widths
+        else:
+            detailRatios = [(pagesize[0] - 1.5*inch) / len(cols)] * len(cols)
+
         det_elements = []
         for i in range(len(cols)):
-            # TODO: It appears that I need to impose some sort of default width in the ModelColumn objects.
-            # That would be useful for both QTableView and for reports.
-            det_elements.append(ObjectValue(attribute_name=cols[i].attr, top=0, left=i*2.54*cm))
+            wf = u
+            style = {}
+            if cols[i].type_ is decimal.Decimal:
+                style['alignment'] = TA_RIGHT
+                wf = sz
+            else:
+                style['wordWrap'] = True
+            
+            det_elements.append(ObjectValue(
+                        attribute_name=cols[i].attr, 
+                        get_value=wf(cols[i].attr), 
+                        top=0, 
+                        left=sum([detailRatios[j] for j in range(i)]), 
+                        width=detailRatios[i], 
+                        style=style))
 
         class GeraldoBS(GeraldoTemplate(self, pagesize)):
             class band_detail(DetailBand):
-                height=0.5*cm
+                height=0.2*inch
+                auto_expand_height = True
                 elements = det_elements
 
         bs = GeraldoBS(queryset = self.data())
@@ -447,15 +456,17 @@ class AccountBalanceListReport(PyHaccReport):
     def objectConverter(self):
         return lambda x: x.accounts_id
 
-    def geraldo(self, outfile, pagesize):
+    @property
+    def report_title(self):
+        return self.title_shell.format(self)
+
+    def geraldo(self, outfile, pagesize, **kwargs):
         from geraldo import Report, DetailBand, ObjectValue,FIELD_ACTION_COUNT, FIELD_ACTION_SUM,  ReportGroup,  ReportBand, SystemField, BAND_WIDTH
         from geraldo.utils import cm
         from geraldo.generators import PDFGenerator
         from reportlab.lib.enums import TA_CENTER, TA_RIGHT
 
         class GeraldoBS(GeraldoTemplate(self, pagesize)):
-            title = self.title_shell.format(self)
-
             class band_detail(DetailBand):
                 height=0.5*cm
                 elements = [
@@ -541,11 +552,11 @@ class TransactionsByAccount(PyHaccReport):
     >>> from pyhacc import MemorySource
     >>> Session = MemorySource(datetime.date(2011,3,31))
     >>> s = Session()
-    >>> rpt = sessionedObjectInit(s,TransactionsByAccount,account='Cash')
+    >>> rpt = sessionedObjectInit(s,TransactionsByAccount,account='Cash', begin_date='jan 1 2011', end_date='dec 31 2011')
     >>> for x in rpt.data():
     ...     print "{0.date:%Y.%m.%d} {0.payee:10s} {0.memo:10s} {0.sum:>8.2f}".format(x)
     2011.03.31 Giant      Groceries    -11.24
-    >>> rpt = sessionedObjectInit(s,TransactionsByAccount,account='Checking')
+    >>> rpt = sessionedObjectInit(s,TransactionsByAccount,account='Checking', begin_date='jan 1 2011', end_date='dec 31 2011')
     >>> for x in rpt.data():
     ...     print "{0.date:%Y.%m.%d} {0.payee:15s} {0.memo:15s} {0.sum:>8.2f}".format(x)
     2011.01.02 ACME Inc        Payroll           400.00
@@ -565,7 +576,7 @@ class TransactionsByAccount(PyHaccReport):
     2011.03.27 ACME Inc        Payroll           400.00
     2011.03.31 Central Bank    Mortgage         -550.00
     2011.04.03 ACME Inc        Payroll           400.00
-    >>> rpt = sessionedObjectInit(s,TransactionsByAccount,account='Checking', group_payee=True)
+    >>> rpt = sessionedObjectInit(s,TransactionsByAccount,account='Checking', group_payee=True, begin_date='jan 1 2011', end_date='dec 31 2011')
     >>> for x in rpt.data():
     ...     print "{0.payee:15s} {0.sum:>8.2f}".format(x)
     ACME Inc         5600.00
@@ -659,15 +670,15 @@ class TransactionList(PyHaccReport):
     >>> from pyhacc import MemorySource
     >>> Session = MemorySource(datetime.date(2011,3,31))
     >>> s = Session()
-    >>> rpt = sessionedObjectInit(s,TransactionList,type_='Asset')
+    >>> rpt = sessionedObjectInit(s,TransactionList,type_='Asset', begin_date='jan 1 2011', end_date='dec 31 2011')
     >>> for x in rpt.data():
     ...     print "{0.date:%Y.%m.%d} {0.payee:15s} {0.memo:15s} {0.sum:>8.2f}".format(x)
     2011.01.02 ACME Inc        Payroll           400.00
     2011.01.09 ACME Inc        Payroll           400.00
     2011.01.16 ACME Inc        Payroll           400.00
     2011.01.23 ACME Inc        Payroll           400.00
-    2011.01.30 Central Bank    Mortgage         -550.00
     2011.01.30 ACME Inc        Payroll           400.00
+    2011.01.30 Central Bank    Mortgage         -550.00
     2011.02.06 ACME Inc        Payroll           400.00
     2011.02.13 ACME Inc        Payroll           400.00
     2011.02.20 ACME Inc        Payroll           400.00
@@ -680,7 +691,7 @@ class TransactionList(PyHaccReport):
     2011.03.31 Central Bank    Mortgage         -550.00
     2011.03.31 Giant           Groceries         -11.24
     2011.04.03 ACME Inc        Payroll           400.00
-    >>> rpt = sessionedObjectInit(s,TransactionList,begin_date='mar 1')
+    >>> rpt = sessionedObjectInit(s,TransactionList, begin_date='mar 1 2011', end_date='dec 31 2011')
     >>> for x in rpt.data():
     ...     print "{0.date:%Y.%m.%d} {0.account:15s} {0.payee:15s} {0.memo:15s} {0.sum:>8.2f}".format(x)
     2011.03.01 Checking        Central Bank    Mortgage         -550.00
@@ -725,7 +736,7 @@ class TransactionList(PyHaccReport):
             q=q.filter(Transactions.date>=self.begin_date)
         if self.end_date is not None:
             q=q.filter(Transactions.date<=self.end_date)
-        q = q.order_by(Transactions.date)
+        q = q.order_by(Transactions.date, Transactions.payee)
         rows = q.all()
         for r in rows:
             dc_object(r,"sum")
@@ -733,31 +744,23 @@ class TransactionList(PyHaccReport):
         return rows
 
     def columns(self):
-        return [ModelColumn("account",str,label="Account"),
-            ModelColumn("date",datetime.date,label="Date"),
-            ModelColumn("reference",str,label="Reference"),
-            ModelColumn("payee",str,label="Payee"),
-            ModelColumn("memo",str,label="Memo"),
-            ModelColumn("debit",decimal.Decimal,label="Debit"),
-            ModelColumn("credit",decimal.Decimal,label="Crebit")]
+        return [ModelColumn("account",str,label="Account",width=20),
+            ModelColumn("date",datetime.date,label="Date",width=15),
+            ModelColumn("reference",str,label="Reference",width=12),
+            ModelColumn("payee",str,label="Payee",width=40),
+            ModelColumn("memo",str,label="Memo",width=40),
+            ModelColumn("debit",decimal.Decimal,label="Debit",width=12),
+            ModelColumn("credit",decimal.Decimal,label="Crebit",width=12)]
 
     def objectConverter(self):
         return lambda x: x.tid
 
-    def geraldo(self, outfile, pagesize):
-        from geraldo import Report, DetailBand, ObjectValue,FIELD_ACTION_COUNT, FIELD_ACTION_SUM,  ReportGroup,  ReportBand, SystemField, BAND_WIDTH
-        from geraldo.utils import cm
-        from geraldo.generators import PDFGenerator
-        from reportlab.lib.enums import TA_CENTER, TA_RIGHT
-
-        if pagesize is None:
-            pagesize = "landscape_letter"
-        sz = SuppressedZero
-        def u(attr):
-            return lambda instance: getattr(instance, attr).replace("&", "&amp;")
-        def fd(attr):
-            return lambda instance: getattr(instance, attr).strftime("%x")
+    @property
+    def page_size(self):
+        return "landscape_letter"
 
+    @property
+    def report_title(self):
         my_title = "Transactions"
         if self.begin_date is not None and self.end_date is not None:
             my_title = "Transactions {0.begin_date:%x} to {0.end_date:%x}".format(self)
@@ -768,25 +771,7 @@ class TransactionList(PyHaccReport):
 
         if self.type_obj is not None:
             my_title += " (Type {0.type_name})".format(self)
-
-        class GeraldoBS(GeraldoTemplate(self, pagesize)):
-            title = my_title
-
-            class band_detail(DetailBand):
-                height=0.5*cm
-                auto_expand_height=True,
-                elements = [
-                            ObjectValue(attribute_name='account', top=0, left=1*cm),
-                            ObjectValue(attribute_name='date', get_value=fd("date"), top=0, left=4*cm),
-                            ObjectValue(attribute_name='reference', top=0, left=6*cm),
-                            ObjectValue(attribute_name='payee', get_value=u("payee"), top=0, left=7.5*cm, width=4.25*cm, style={"wordWrap": True}),
-                            ObjectValue(attribute_name='memo', top=0, left=12*cm, style={"wordWrap": True}),
-                            ObjectValue(attribute_name='debit', get_value=sz("debit"), display_format="%s", top=0, left=15*cm, style={'alignment': TA_RIGHT}),
-                            ObjectValue(attribute_name='credit', get_value=sz("credit"), display_format="%s", top=0, left=17*cm, style={'alignment': TA_RIGHT})
-                            ]
-
-        bs = GeraldoBS(queryset = self.data())
-        bs.generate_by(PDFGenerator, filename=outfile)
+        return my_title
 
 class ProfitAndLoss(AccountBalanceListReport):
     """
@@ -794,7 +779,7 @@ class ProfitAndLoss(AccountBalanceListReport):
     >>> from pyhacc import MemorySource
     >>> Session = MemorySource(datetime.date(2011,3,31))
     >>> s = Session()
-    >>> rpt = sessionedObjectInit(s,ProfitAndLoss,begin_date='mar 1', end_date='mar 31')
+    >>> rpt = sessionedObjectInit(s,ProfitAndLoss,begin_date='mar 1 2011', end_date='mar 31 2011')
     >>> for x in rpt.data():
     ...     print "{0.accounts_name:15s} {0.amount:>8.2f}".format(x)
     Day Job         -1600.00
@@ -861,7 +846,7 @@ class IntervalPL(PyHaccReport):
     >>> from pyhacc import MemorySource
     >>> Session = MemorySource(datetime.date(2011,3,31))
     >>> s = Session()
-    >>> rpt = sessionedObjectInit(s,IntervalPL,end_date='jun 30', interval_length = 3)
+    >>> rpt = sessionedObjectInit(s,IntervalPL,end_date='jun 30 2011', interval_length = 3)
     >>> for x in rpt.data():
     ...     print "{0.accounts_name:15s} {0.amount0:>8.2f} {0.amount1:>8.2f} {0.amount2:>8.2f}".format(x)
     Day Job         -5200.00 -5200.00  -400.00
@@ -945,7 +930,12 @@ class IntervalPL(PyHaccReport):
     def tableExtensionId(self):
         return "DataTable_{0}column".format(self.interval_count)
 
-    def geraldo(self, outfile, pagesize):
+    @property
+    def report_title(self):
+        dates = list(intervals(self.end_date, self.interval_count, self.interval_length))
+        return "{0.name} for {1:%x} - {2:%x}".format(self, dates[0][0], dates[-1][1])
+
+    def geraldo(self, outfile, pagesize, **kwargs):
         from geraldo import Report, DetailBand, ObjectValue, Label, FIELD_ACTION_COUNT, FIELD_ACTION_SUM,  ReportGroup,  ReportBand, SystemField, BAND_WIDTH
         from geraldo.utils import cm
         from geraldo.generators import PDFGenerator
@@ -971,8 +961,6 @@ class IntervalPL(PyHaccReport):
                                 style={'fontName': 'Helvetica-Bold', 'fontSize': 12}) for i in range(self.interval_count)]
 
         class GeraldoInterval(GeraldoTemplate(self, pagesize)):
-            title = cgi.escape("{0.name} for {1:%x} - {2:%x}".format(self, dates[0][0], dates[-1][1]))
-
             class band_detail(DetailBand):
                 height=0.7*cm
                 elements = [
